package de.ozgcloud.operator.common.elasticsearch;

import java.time.Duration;
import java.time.temporal.ChronoUnit;

import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.testcontainers.elasticsearch.ElasticsearchContainer;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.json.jackson.JacksonJsonpMapper;
import co.elastic.clients.transport.rest_client.RestClientTransport;

public class ElasticsearchTestClient {
	
	private static final String IMAGE_NAME = "docker.elastic.co/elasticsearch/elasticsearch:8.11.3";
	private static final String USER = "elastic";
	private static final String PASSWORD = "s3cret";
	private static final int PORT = 9200;
	private static final String SCHEME = "https";
	private static final Duration STARTUP_TIMEOUT = Duration.of(2, ChronoUnit.MINUTES);
	 
	public static final ElasticsearchContainer ELASTICSEARCH_CONTAINER = new ElasticsearchContainer(IMAGE_NAME)
			.withExposedPorts(PORT)
			.withPassword(PASSWORD)
			.withStartupTimeout(STARTUP_TIMEOUT);

	public static ElasticsearchClient create() {
		var transport = new RestClientTransport(buildRestClient(), new JacksonJsonpMapper());
		return new ElasticsearchClient(transport);
	}
	
	private static RestClient buildRestClient() {
		var host = new HttpHost("localhost", ELASTICSEARCH_CONTAINER.getMappedPort(PORT), SCHEME);
		var credentialsProvider = buildCredentialsProvider();
		var builder = RestClient.builder(host);

		builder.setHttpClientConfigCallback(clientBuilder -> {
			clientBuilder.setSSLContext(ELASTICSEARCH_CONTAINER.createSslContextFromCa());
			clientBuilder.setDefaultCredentialsProvider(credentialsProvider);
			return clientBuilder;
		});
//		builder.setNodeSelector(INGEST_NODE_SELECTOR);
//        final ObjectMapper mapper = new ObjectMapper();
//        mapper.setPropertyNamingStrategy(PropertyNamingStrategies.SNAKE_CASE);
//        mapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);
		return builder.build();
	}
	
	private static BasicCredentialsProvider buildCredentialsProvider() {
		var credentialsProvider = new BasicCredentialsProvider();
		credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(USER, PASSWORD));
		return credentialsProvider;
	}

}
